function h = channel_body(N,d,position,floor)
%
% Written by Andrew Fort (IMEC, Belgium. April 12, 2005)
%
% PROTOTYPE
%
% h = channel_body(N,d,position,floor)
%
% INPUTS
%
% N = number of channels to generate
% d = distance between tx and rx (meters)
% position = Position on body ('front','side','back', or 'front_front')
%
% OUTPUTS
%
% h = N randomly generated channel responses 
% 
% DESCRIPTION
%
% h is an N by M matrix. N is the number of different
% randomly generated channel response. M is the number of filter taps in 
% a single channel realization. Taps are always separated by 0.5 ns.
%
% Results were determined emperically using a sophisticated finite difference 
% time domain simulation and an anatomically correct body area model.
% General model parameters were confirmed through actual measurements.
% 
% 'front', 'side', and 'back' refer to diffraction around body.
% 'front_front' refers to communication along front of body.

% Load emperically derived path loss model
% P0 = reference path loss (dB)
% d0 = reference distance (m)
% n  = pathloss exponent
if(strcmp(position,'front_front'))
    load pathloss_par_body_front.mat
else
    load pathloss_par_body_v7.mat;
end

% Load empirically derived amplitude distributions
% Mbody = Mean amplitude for each bin (initial cluster)
% Cbody = Covariance matrix for each bin (initial cluster)
% Mground = Mean amplitude for each bin (ground reflection cluster)
% Cground = Covariance matrix for each bin (ground reflection cluster)
% body_ground_iat = Average inter arrival time between body and ground clusters (s)
% binlen = length of one bin (s)
switch(position)
    case 'front'
        load front_par_body_v7.mat;
    case 'front_front'
        load front_par_body_front.mat;
    case 'side'
        load side_par_body_v7.mat;
    case 'back'
        load back_par_body_v7.mat;
    otherwise
        error('Position parameter must be ''front'', ''side'', ''back'', or ''front_front''');
end

% The channel model is created in the log domain and then
% converted to the linear domain.

% Generate correlated normal variables representing each bin 
% in the initial cluster of components diffracting around the body.
hbody = randn(N,size(Cbody,2));
hbody = hbody*chol(Cbody) + repmat(Mbody,size(hbody,1),1);

% Apply path loss model around body.
hbody = hbody - (P0 + 10*n*log10(d/d0)); 

% Generate correlated normal variables representing each bin
% in the second cluster of components reflecting off of the ground.
% Then adjust for different kinds of floor materials
if(~strcmp(floor,'none'))
    hground = randn(N,size(Cground,2));
    hground = hground*chol(Cground) + repmat(Mground,size(hground,1),1);
end
switch(floor)
    case 'PEC'
        hground = hground - 6.0;% 6 dB adjustment
        
    case 'concrete'
        % The measurement was made on concrete so no adjustment needed.
        ;
        
    case 'ground'
        hground = hground -4.9; % 4.9 dB adjustment
        
    case 'none'
        hground = zeros(N,size(Cground,2)) - inf; % Set this cluster of components to 0
        
    otherwise
        error('The floor argument must be ''PEC'', ''concrete'', ''ground'', or ''none''');
end

% In general, the time of arrival of the second cluster depends on the 
% heights of the antennas on the torso, and the position around the body.
% To simplify this, we used the average time between the first and second
% clusters, body_ground_iat, extracted along the front, side, and back
% of the body.

% Calculate number of bins between first and second cluster
icbin = round(body_ground_iat/binlen)-size(hbody,2);

% Create matrix of channels in the correct size:
% N by (Length of first cluster + icbin + length of second cluster)
h = zeros(N,size(hbody,2)+icbin+size(hground,2));

% Convert from dB to linear, and put the body and ground
% clusters at the correct times.
h(:,1:size(hbody,2)) = 10.^(hbody./10);
h(:,size(hbody,2)+1:size(hbody,2)+icbin) = 0;
h(:,size(hbody,2)+icbin+1:end) = 10.^(hground./10);

% Convert to tap amplitudes and apply uniform random phase
h = sqrt(h).*exp(j*2*pi*rand(size(h)));
