function h = uwbchannel(scenario,Ts,type,N,d)
%
% Written by Andrew Fort (June 12, 2004)
%
% PROTOTYPES
%
% h = uwbchannel('154a',Ts,type,N)
% h = uwbchannel('BAN',Ts,type,N)
% h = uwbchannel('BAN_floor',Ts,type,N,d)
% h = uwbchannel('BAN_indoor',Ts,type,N,d)
%
% INPUTS
%
% scenario - '154a' : 802.15.4a models 1-8
%            'BAN'  : Body area network (outdoor environment)
%            'BAN_floor' : Body area network (outdoor environment with
%            floor reflection)
%            'BAN_indoor' : Body area network (indoor environment)
% Ts - Sampling rate (in ns). See below.
% type - For 154a, this is a number between 1-8 corresponding to the
%        different channel environments (see below). For 'BAN', 'BAN_floor' and 
%        'BAN_indoor', this can be 'front', 'side' or 'back'.
% N - Number of channel realizations to generate
% d - Distance around body (m). This is needed to specify the strength of the
%     initial diffracting component relative to reflected waves (from the floor or 
%     surrounding environment). The larger this value, the stronger ground
%     and indoor reflections are relative to the diffracting components.
%
% OUTPUTS
%
% h - an L by N matrix of N channels of length L*Ts. Each element
% represents the complex magnitude of a resolvable multipath component.
%
% DESCRIPTION
%
% N channels are randomly generated according to the specified scenario. 
% Each channel realization is normalized to unit energy. 
%
% The sampling rate for each model depends on the bandwidth of the
% measurement as follows:
%
% BAN models: Ts = 0.5 ns resolution.
% IEEE 802.15.4a models 1-8: Ts = 0.125 ns resolution.
%
% This function will automatically resample the channel using matlab's
% "resample" function to a new sampling period that is an integer multiple
% larger than the original sampling period. Specifying a smaller sampling
% period will result in error since each model is limited by the
% measurement resolution.
%
% The IEEE 802.15.4a channel models are defined as follows:
%
% 1 Residential LOS
% 2 Residential NLOS
% 3 Office LOS
% 4 Office NLOS
% 5 Outdoor LOS
% 6 Outdoor NLOS
% 7 Industrial LOS
% 8 Industrial NLOS
%

% Check number of parameters
if( (nargin < 4) | (nargin > 5) )
    error('Channel model expects between 4 and 5 parameters');
end

% Check that scenario is a valid string
if(~isstr(scenario))
    error('First parameter should be a string specifying the channel scenario');
end

switch(scenario)
    
    case '154a'
        
        % Check parameters
        if(~isnumeric(Ts))
            error('Parameters Ts must be a number');
        elseif(~isinteger(type))
            error('Type parameter must be an integer number for 154a');
        elseif((type < 1) | (type > 8))
            error('Channel type must be an integer between 1 and 8');
        elseif(~isinteger(N))
            error('N parameter must be an integer');
        elseif(N < 1)
            error('N must be >= 1');
        end
        
        % Generate specified channels
        h = genchan_ieee154a(type,N,12);
        Tschan = 0.125; % 0.125ns is the resolution of the ieee model.
        
    case 'BAN'
        % Check parameters
        if(~isnumeric(Ts))
            error('Parameters Ts must be a number');
        elseif(~isstr(type))
            error('Type parameter must be a string');
        elseif(~isinteger(N))
            error('N parameter must be an integer');
        elseif(N < 1)
            error('N must be >= 1');
        end
        
        % Call function channel_body to generate N channels
        h = channel_body(N,d,type,'none').';
        Tschan = 0.5;
        
        % Normalize to unit energy.
        %h = h./repmat(sqrt(sum(abs(h(:,:)).^2)),size(h,1),1);
        
    case 'BAN_floor'
        
        % Check parameters
        if(~isnumeric(Ts))
            error('Parameters Ts must be a number');
        elseif(~isstr(type))
            error('Type parameter must be a string');
        elseif(~isinteger(N))
            error('N parameter must be an integer');
        elseif(N < 1)
            error('N must be >= 1');
        elseif(~isnumeric(d));
            error('d parameter must be an integer');
        elseif(d <= 0)
            error('d must be greater than 0');
        end
        
        % Call function channel_body to generate N channels
        h = channel_body(N,d,type,'concrete').';
        Tschan = 0.5;
        
    case 'BAN_indoor'
        % Check parameters
        if(~isnumeric(Ts))
            error('Parameters Ts must be a number');
        elseif(~isstr(type))
            error('Type parameter must be a string');
        elseif(~isinteger(N))
            error('N parameter must be an integer');
        elseif(N < 1)
            error('N must be >= 1');
        elseif(~isnumeric(d));
            error('d parameter must be an integer');
        elseif(d <= 0)
            error('d must be greater than 0');
        end
        
        % Call function channel_body to generate N channels
        h_body = channel_body(N,d,type,'none');
        Tschan = 0.5;
        
        % Generate reflections
		if(strcmp(type,'front_front'))
            type = 'front';
		end
		for n = 1:N
            h_reflections(n,:) = sqrt(channel_reflections([type '_par_indoor'],1));
            h_reflections(n,:) = h_reflections(n,:).*exp(j*2*pi*rand(1,size(h_reflections,2)));
		end
        
        % Add body and reflection channels together
        h_reflections(:,1:size(h_body,2)) = ...
            h_reflections(:,1:size(h_body,2)) + h_body;
        
        % Normalize to unit energy
        h = h_reflections.';
        %h = h./repmat(sqrt(sum(abs(h(:,:)).^2)),size(h,1),1);
        Tschan = 0.5;
        
    otherwise
        error('uwbchannel scenario must be ''154a'', ''BAN'', or ''BAN_indoor''.');
        
end

% If specified, resample the channel.
if(Ts > Tschan)
    if(rem(Ts,Tschan))
        error(['Ts must be an integer multiple of ' num2str(Tschan) ' for this scenario']);
    end
    
    % Resample using matlab resample operation
    h = resample(h,1,Ts/Tschan);
    
    % Re-normalize to unit energy
    for c = 1:size(h,2)
        h(:,c) = h(:,c)/sqrt(h(:,c)' * h(:,c) );
    end
    
elseif(Ts < Tschan)
    error('Specified sampling period is higher than channel resolution.');

end

% ****************************************************************
% ****************** Sub-functions *******************************
% ****************************************************************

% Function to check if argument is an integer
function b = isinteger(x)
if(~isnumeric(x))
    b=0;
elseif( (x-round(x)) ~= 0)
    b=0;
else
    b=1;
end

% Generate channels from ieee802154a standard (version 9). 
% Written by Sun Xu, Kim Chee Wee, B. Kannan & Francois Chin on 22/02/2005 
% Copied, pasted, and commented by Andrew Fort.
function h = genchan_ieee154a(cm_num,num_channels,state)

% initialize state of function for repeatability
randn('state',state); 
rand('state',state);     

% get channel model params based on this channel model number
[Lam,Lmean,lambda_mode,lambda_1,lambda_2,beta,Gam,gamma_0,Kgamma, ...
 sigma_cluster,nlos,gamma_rise,gamma_1,chi,m0,Km,sigma_m0,sigma_Km, ...
 sfading_mode,m0_sp,std_shdw,kappa,fc,fs] = uwb_sv_params_15_4a( cm_num );

% get a bunch of realizations (impulse responses)
ts = 1/fs;  % sampling frequency
[h_ct,t_ct,t0,np] = uwb_sv_model_ct_15_4a(Lam,Lmean,lambda_mode,lambda_1, ...
    lambda_2,beta,Gam,gamma_0,Kgamma,sigma_cluster,nlos,gamma_rise,gamma_1, ...
    chi,m0,Km,sigma_m0,sigma_Km,sfading_mode,m0_sp,std_shdw,num_channels,ts);

% change to complex baseband channel
h_ct_len = size(h_ct, 1);
phi = zeros(h_ct_len, 1);
for k = 1:num_channels
    phi = rand(h_ct_len, 1).*(2*pi);
    h_ct(:,k) = h_ct(:,k) .* exp(phi .* i);
end

% now reduce continuous-time result to a discrete-time result
[hN,N] = uwb_sv_cnvrt_ct_15_4a( h_ct, t_ct, np, num_channels, ts );
if N > 1,
  h = resample(hN, 1, N);  % decimate the columns of hN by factor N
else
  h = hN;
end

% add the frequency dependency
[h]= uwb_sv_freq_depend_ct_15_4a(h,fc,fs,num_channels,kappa);
